/*
 *   Copyright (c) 2003 David Parsons. All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. All advertising materials mentioning features or use of this
 *     software must display the following acknowledgement:
 *     
 *   This product includes software developed by David Parsons
 *   (orc@pell.chi.il.us)
 *
 *  4. My name may not be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *     
 *  THIS SOFTWARE IS PROVIDED BY DAVID PARSONS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL DAVID
 *  PARSONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdarg.h>
#include <errno.h>
#ifdef __FreeBSD__
# include <stdlib.h>
#else
# include <malloc.h>
#endif
#include <string.h>

/*
 * safe malloc & string handling functions.
 */

extern void e_to_sys(int);
extern void err(char *fmt, ...);


char *
xstrdup(char *string)
{
	char *ret = strdup(string);

	if (ret == 0) {
	    e_to_sys(errno);
	    err("could not duplicate ``%s''.", string);
	}
	return ret;
}


void *
xmalloc(const int size)
{
	void *ret = malloc(size);

	if (ret == 0) {
		e_to_sys(errno);
		err("could not allocate %d byte%s.", size, (size==1)?"":"s");
	}
	return ret;
}


void *
xrealloc(void *bfr, const int size)
{
	void *ret = realloc(bfr, size);

	if (ret == 0) {
		e_to_sys(errno);
		err("could not reallocate %d byte%s", size, (size==1)?"":"s");
	}
	return ret;
}


char *
concatenate(const char *first, ...)
{
	int size;
	char *arg;
	char *ret;

	va_list rest;
	ret = xmalloc(size = strlen(first)+1);

	strcpy(ret, first);

	for (va_start(rest, first); arg = va_arg(rest,char*); ) {
		size += strlen(arg);
		ret = xrealloc(ret, size);
		strcat(ret, arg);
	}
	va_end(rest);

	return ret;
} /* concatenate */
